/*
 * Copyright (C) 2005 Cesar Kallas
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * E-mail: cesarkallas@gmx.net
 * http://cesarkallas.soulivre.org
 *
 */

package estella.lib;
import  estella.lib.Log;
import  estella.lib.Constantes;

import  java.net.Socket;
import  java.net.ServerSocket;
import  java.io.IOException;
import  java.io.DataInputStream;
import  java.io.DataOutputStream;
import  java.io.BufferedReader;
import  java.io.InputStreamReader;
import  java.net.SocketException;
import  java.net.SocketTimeoutException;
import  java.lang.String;
import  java.lang.Thread;
/**
* Funcoes para gerenciar conexoes de saida
* @author Cesar Kallas - cesarkallas@gmx.net
* @since 22/11/2005
*/
public class Conexao extends Thread
{
  private Log LOG;
  private Constantes CON;
  private Socket socket;
	private DataInputStream receive;
	private DataOutputStream send;
  private BufferedReader bufferSocket;
  private String host;
  private int porta;

  /**
	* Instancia uma nova conexao
	* @param host String com o host no formato host.com ou 127.0.0.1
	* @param porta Inteiro contendo um numero de porta
	*/
  public Conexao(String host, int porta)
  {
    this.host = host;
    this.porta = porta;
  }
  
  public Conexao(Socket sckt)
  {
    this.socket = sckt;
		this.porta = socket.getPort();
		this.host = socket.getInetAddress().getHostAddress();
  }

  /**
	* Cria um socket cliente, setando os parametros necessarios
	* @return true ou false / se a conexao foi criada e se modo leitura e gravacao estao prontos
	*/  
  public boolean inicia()
	{
    CON = new Constantes();
    LOG = new Log("Conexao");
    
    // Cria um novo socket, se ele nao foi passado na criacao da classe
    if(socket == null)
      try {
        socket = new Socket(host, porta);
      }
      catch(IOException FalhaConectar) {
        LOG.falha(FalhaConectar.getMessage() + " @ " + host + ":" + porta);
        return false;
      }
      catch(SecurityException Firewall) {
        LOG.atencao(Firewall.getMessage()  + " @ " + host + ":" + porta);
        return false;
      }
      catch(Exception FalhaGeral) {
        LOG.falha(FalhaGeral.getMessage()  + " @ " + host + ":" + porta);
        return false;
      }
      
    // Inicia leitura e gravacao no socket
    try {
      socket.setTcpNoDelay(CON.SOCKET_TCPNODELAY);
      receive = new DataInputStream(socket.getInputStream());
      send = new DataOutputStream(socket.getOutputStream());
      //LOG.info("Cliente conectado: " + socket);
      return true;
		}
    catch(IOException FalhaCanal) {
      LOG.falha(FalhaCanal.getMessage());
      fecha();
      return false;
    }
  }
  
  /**
	* Escreve dados no socket
	* @param msg - String com a mensagem a ser escrita no socket
	* @return true ou false
	*/
  public boolean envia(String msg)
  {
    try {
			msg = msg;
			byte[] data = msg.getBytes();
      send.write(data);
      send.flush();
      //LOG.info("mensagem(" + msg.length() + "b)  --> " + socket);
      return true;
    }
    catch(Exception EnviarMsg) {
      LOG.falha(" falha o enviar msg para o host " + host + EnviarMsg.getMessage());
			fecha();
      return false;
    }
  }
  
	 /**
	* Escreve dados no socket
	* @param arquivo - arquivo em bytes para ser escritor no socket
	* @return true ou false
	*/
  public boolean envia(byte[] arquivo)
  {
		// 0 = the start offset in the data.
    try {
      send.write(arquivo, 0, arquivo.length);
      //LOG.info("enviando arquivo(" + arquivo.length + "b)  --> " + socket);
      return true;
    }
    catch(Exception EnviarArquivo) {
      LOG.falha(EnviarArquivo.getMessage() + socket);
      return false;
    }
  }
	
  /**
	* Retorna o host da conexao
	* @return host
	*/
  public String host()
  {
    return host;
  }
  
	/**
	* Retorna o ip no qual essa conexao se conectou (ip do servidor local)
	* @return string 
	*/	
	public String ipLocal()
	{
		return socket.getLocalAddress().getHostAddress();
	}
	
  /**
	* Retorna a porta da conexao
	* @return porta
	*/
  public int porta()
  {
    return porta;
  }
  
  /**
	* Timeout - seta o tempo de espera para resposta da conexao
	* @param tempo (s) - tempo em segundos
	* @return true ou false
	*/
  public boolean setTimeout(int tempo)
  {
    try {
      socket.setSoTimeout(tempo);
      return true;
    }
    catch (IOException socketTimeout) {
			LOG.falha(socketTimeout.getMessage());
      return false;
    }
  }

  /**
	* Timeout - retorna o tempo de espera para resposta da conexao
	* @param tempo (s) - tempo em segundos
	* @return true ou false
	*/
  public int getTimeout()
  {
    try {
      return socket.getSoTimeout();
    }
    catch (IOException socketTimeout) {
			LOG.falha(socketTimeout.getMessage());
      return -1;
    }
  }
  
  /**
	* Recebe dados do socket (com timeout, vide classe constantes)
	* @return msg - String com a mensagem recebida ou null - Problema na recepcao de dados
	*/
  public String recebeV()
  {
    String msg = new String();
		char[] teste;
    try {
      bufferSocket = new BufferedReader(new InputStreamReader(socket.getInputStream()));
			
      try {
				msg = bufferSocket.readLine();
      }
      catch (SocketTimeoutException socketTimeout) {
        envia(CON.ESTELLA + '\r'+'\n' + CON.CONNECTION_TIMEOUT + '\r'+'\n');
        fecha();
        return null;
      }
      
      //LOG.info(socket + " <--  mensagem(" + msg.length() + "b)");
      return(msg);
    }
    catch(Exception ReceberMsg) {
      LOG.falha(ReceberMsg.getMessage() + " : " + socket);
      fecha();
      return null;
    }
  }
  
	/**
	* Recebe dados do socket (com timeout, vide classe constantes)
	* @return byte[] - array com bytes
	* @see Constantes
	*/
	public String recebe()
	{
		Tempo t = new Tempo();
		String mensagem = new String();
		
    try {
			while(true) {
				byte[] data = new byte[receive.available()];
				receive.readFully(data);
				if(data.length > 0) {
					mensagem = new String(data);
					break;
				}
				if(socket.isClosed()) {
					fecha();
					return null;
				}
				sleep(1);
			}return mensagem;
    }			
    catch (SocketTimeoutException socketTimeout) {
      envia(CON.ESTELLA + "\r\n" + CON.CONNECTION_TIMEOUT + "\r\n");
      fecha();
      return null;
    }
		catch(Exception ReceberMsg) {
      LOG.falha(ReceberMsg.getMessage() + " : " + socket);
      fecha();
      return null;
    }
	}
	
	public String recebeSEO()
	{
		try {
			byte[] data = new byte[receive.available()];
			receive.readFully(data);
			return new String(data);
		}
		catch(Exception X) {
			return null;
		}
	}	
		
	/**
	* Status da conexao
	* @return socket da conexao (podendo retornar null se desconectado)
	*/
  public Socket status()
  {
    return socket;
  }
    
  /**
	* Fecha conexao cliente, incluindo os canais de comunicacao (leitura / escrita)
	* @return true ou false
	*/
  public boolean fecha()
  {
    String sck_info;
    try {
      sck_info = "" + socket;
      receive.close();
      receive = null;
      send.close();
      send = null;
      socket.close();
      socket = null;
      LOG.info("Cliente desconectado: " + sck_info);
      return true;
    }
    catch(IOException FalhaFecharSocket) {
      LOG.falha(FalhaFecharSocket.getMessage());
      return false;
    }
  }
}
