/*
 * Copyright (C) 2005 Cesar Kallas
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * E-mail: cesarkallas@gmx.net
 * http://cesarkallas.soulivre.org
 *
 */
 
package estella.lib;
import  estella.lib.Log;
import  estella.lib.Lista;

import  java.util.List;
import  java.util.LinkedList;
import  java.util.Collections;
import  java.util.Iterator;

/**
* Gerencia uma lista do tipo Conexao
* Essa classe depende da classe de Lista, no qual gerencia uma Lista de objetos
* @author Cesar Kallas - cesarkallas@gmx.net
* @since 01/12/2005
* @see Lista
*/
public class ConexaoLista extends Lista
{
 
	public ConexaoLista()
	{
		super();
	}
	
  public ConexaoLista(List LISTA)
  {
    super(LISTA);
  }
  
  /**
	* Verifica se existe um host (ip / nome computador) na lista
	* @param host a ser verificado
	* @return true ou false
	*/
  public boolean existe(String host)
  {
    Iterator i;
    try {
      synchronized(LISTA) {
        i = LISTA.iterator();
        while (i.hasNext()) {    
          Conexao conexao = (Conexao) i.next();
          if(conexao.host().equals(host))
            return true;
        }
      }
      return false;
    }
    catch(Exception FalhaConsultaObj) {
      LOG.info(FalhaConsultaObj.getMessage());
      return false;
    }
  }

	/**
	* Verifica se existe um host (ip / nome computador) na lista
	* @param host a ser verificado
	* @return true ou false
	*/
  public boolean desconecta(String host, int porta)
  {
    Iterator i;
    try {
      synchronized(LISTA) {
        i = LISTA.iterator();
        while (i.hasNext()) {    
          Conexao conexao = (Conexao) i.next();
          if(conexao.host().equals(host))
						if(conexao.porta() == porta)
							if(conexao.fecha()) {
								remove(conexao);
								return true;
							}
        }
      }
      return false;
    }
    catch(Exception FalhaConsultaObj) {
      LOG.falha(" ao remover host da lista "  + FalhaConsultaObj.getMessage());
      return false;
    }
  }
	
	/**
	* Envia a mensagem para todos da lista
	* @param msg String com a mensagem
	* @return true ou false
	*/
	public boolean envia(String msg)
  {
    Iterator i;
    try {
			synchronized(LISTA) {
				i = LISTA.iterator();
				while (i.hasNext()) {
					Conexao conexao = (Conexao) i.next();
					if(conexao.status().isConnected()) {
						if(!conexao.status().getInetAddress().getHostAddress().equals(Dados.ip))
						if(!conexao.envia(msg)) {
							conexao.fecha();
							remove(conexao);
						}							
					} else {
						conexao.fecha();
						remove(conexao);
					}
				}
			}
      return true;
		}
    catch(Exception FalhaConsultaObj) {
      LOG.info(FalhaConsultaObj.getMessage());
      return false;
    }
  }
	
	/**
	* Envia a mensagem para todos da lista menos para o host de origem
	* @param hostOrigem string com o host que nao eh pra enviar
	* @param msg String com a mensagem
	* @return true ou false
	*/
	public boolean envia(String msg, String hostOrigem)
  {
    Iterator i;
    try {
			synchronized(LISTA) {
				i = LISTA.iterator();
				while (i.hasNext()) {
					Conexao conexao = (Conexao) i.next();
					if(conexao.status().isConnected()) {
						if(!conexao.status().getInetAddress().getHostAddress().equals(Dados.ip))
						if(!conexao.status().getInetAddress().getHostAddress().equals(hostOrigem))
						if(!conexao.envia(msg)) {
							conexao.fecha();
							remove(conexao);
						}							
					} else {
						conexao.fecha();
						remove(conexao);
					}
				}
			}
      return true;
		}
    catch(Exception FalhaConsultaObj) {
      LOG.info(FalhaConsultaObj.getMessage());
      return false;
    }
  }
	
  public boolean fechaConexoes()
  {
    Iterator i;
    try {
      synchronized(LISTA) {
        i = LISTA.iterator();
        while (i.hasNext()) {    
          Conexao conexao = (Conexao) i.next();
          conexao.fecha();
				}
				return true;
      }
    }
    catch(Exception FalhaConsultaObj) {
      LOG.info(FalhaConsultaObj.getMessage());
      return false;
    }
  }
}
