/*
 * Copyright (C) 2005 Cesar Kallas
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * E-mail: cesarkallas@gmx.net
 * http://cesarkallas.soulivre.org
 *
 */

package estella.lib;
import  estella.lib.Log;
import  estella.lib.Constantes;

import  java.net.Socket;
import  java.net.ServerSocket;
import  java.net.BindException;
import  java.io.IOException;
import  java.io.DataInputStream;
import  java.io.DataOutputStream;

/**
* Funcoes para gerenciar conexoes de entrada
* @author Cesar Kallas - cesarkallas@gmx.net
* @since 22/11/2005
*/
public class ConexaoServidor
{
  private Log LOG;
  private Constantes CON;
  private ServerSocket sv_socket;
  private Socket socket;
	private DataInputStream receive;
	private DataOutputStream send;
  private int porta;

  /**
  * Instancia uma nova conexao
  * @param porta - Constante porta definida pelo usuario ou na classe de constantes
  */
  public ConexaoServidor()
  {
    CON = new Constantes();
    LOG = new Log();
    this.porta = CON.SERVIDOR_PORTA;
  }
  public ConexaoServidor(int porta)
  {
    LOG = new Log();
    this.porta = porta;
  }

  /**
  * Cria um socket servidor, setando os parametros necessarios
  * @return true ou false / se a conexao foi criada e se modo leitura e gravacao estao prontos
  */  
  public boolean inicia()
	{
		while(porta < 65353) {
      try {
        sv_socket = new ServerSocket(porta);
        //sv_socket.setSoTimeout(CON.SOCKET_SERVER_TIMEOUT);
        //LOG.info("Conexao entrante iniciada @ localhost:" + porta);
        return true;
      }
      catch (BindException PortaRecusada) {
        porta++;
        LOG.falha(PortaRecusada.getMessage() + " @ localhost:" + porta);
        return false;
      }
      catch(IOException FalhaIniciar) {
			  LOG.falha(FalhaIniciar.getMessage() + " @ localhost:" + porta);
        return false;
		  }
      catch(SecurityException Firewall) {
        LOG.atencao(Firewall.getMessage()  + " @ localhost:" + porta);
        return false;
      }
    }
    return false;
  }

  /**
  * Escuta conexao de entrada, ate receber uma requisicao
  * @param ServerSocket
  * @return Socket
  */
  public Socket escuta()
  {
    try {
      socket = sv_socket.accept();
      //LOG.info("Nova conexao: " + socket);
      return socket;
    }
    catch(IOException FalhaIniciarConexaoCliente) {
      LOG.falha(FalhaIniciarConexaoCliente.getMessage() + " @ localhost:" + porta);
      return null;
    }
    catch(Exception FalhaIniciarConexaoCliente) {
      LOG.falha(FalhaIniciarConexaoCliente.getMessage() + " @ localhost:" + porta);
      return null;
    }
  }
  
	/**
	* Retorna a porta que o servidor esta escutando
	* @return porta inteiro com a porta que o servidor esta escutando
	*/ 
	public int porta()
	{
		return porta;
	}
	
	public ServerSocket status()
	{
		return sv_socket;
	}
	
	/**
	* Fecha conexao de entrada
	* @param ServerSocket - Socket servidor
	* @return true ou false
	*/
  public boolean fecha()
  {
    try{
      sv_socket.close();
      return true;
    }
    catch(IOException FalhaFecharConexaoServidor) {
      LOG.falha(FalhaFecharConexaoServidor.getMessage() + " @ localhost:" + porta);
      return false;
    }
  }
}
