/*
 * Copyright (C) 2005 Cesar Kallas
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * E-mail: cesarkallas@gmx.net
 * http://cesarkallas.soulivre.org
 *
 */

package estella.lib;
import  estella.lib.*;

import  java.util.List;
import  java.util.LinkedList;
import  java.util.Collections;
import  java.util.Iterator;

/**
* Gerencia uma lista de objetos
* @author Cesar Kallas - cesarkallas@gmx.net
* @since 02/12/2005
*/
public class Lista
{
  protected List LISTA;
  protected Log LOG;
	protected Constantes CON;
		
  public Lista()
  {
		this.LISTA = Collections.synchronizedList(new LinkedList());
    LOG = new Log("Lista");
		CON = new Constantes();
  }
  
  public Lista(List LISTA)
  {
    this.LISTA = LISTA;
    LOG = new Log("Lista");
		CON = new Constantes();
  }
  
  /**
  * Adiciona um objeto a lista
  * @param objeto
  * @return true ou false
  */
  public boolean adiciona(Object objeto)
  {
    try {
      synchronized(LISTA) {
        LISTA.add(objeto);  
        return true;
      }
    }
    catch(Exception FalhaAdicionarObjeto) {
      LOG.falha(FalhaAdicionarObjeto.getMessage());
      return false;
    }  
  }    
  
  /**
  * Remove um objeto da lista
  * @param objeto 
  * @return true ou false
  */
  public boolean remove(Object objeto)
  {
    try {
      synchronized(LISTA) {
        LISTA.remove(objeto);
				return true;
      }
    }
    catch(Exception FalhaObjeto) {
			FalhaObjeto.printStackTrace();
      LOG.falha(" ao remover da lista " + FalhaObjeto.getMessage());
      return false;
    }
  }    

  /**
  * Procura o objeto na lista
  * @param objeto
  * @return index ou -1 (index = inteiro com a localizacao do objeto)
  */ 
  public int existe(Object objeto)
  {
    try {
      synchronized (LISTA) {
        return LISTA.indexOf(objeto);
      }
    }
    catch(Exception FalhaObjeto) {
      LOG.falha(FalhaObjeto.getMessage());
      return -1;
    }
  }

  /**
  * Retorna o objeto especificado pelo index
  * @param index
  * @return objeto
  */ 
  public Object retorna(int index)
  {
    try {
      synchronized (LISTA) {
        return LISTA.get(index);
      }
    }
    catch(Exception FalhaObjeto) {
      LOG.falha(FalhaObjeto.getMessage());
      return null;
    }
  }
  
  /**
  * Tamanho da lista
  * @return inteiro
  */ 
  public int tamanho()
  {
    try {
      synchronized (LISTA) {
        return LISTA.size();
      }
    }
    catch(Exception FalhaObjeto) {
      LOG.falha(FalhaObjeto.getMessage());
      return -1;
    }
  }

  /**
  * Remove todos os objetos da lista
  * @return true ou false
  */
  public boolean limpa()
  {
    try {
      synchronized (LISTA) {
        LISTA.clear();
        return true;
      }
    }
    catch(Exception FalhaObjeto) {
      LOG.falha(FalhaObjeto.getMessage());
      return false;
    }
  }
}    
